@@ -2,7 +2,7 @@
 # the full copyright notices and license terms.
 from trytond.pool import Pool
 
-from . import carrier, checkout, sale
+from . import carrier, sale
 
 __all__ = ['register']
 
@@ -12,10 +12,6 @@ def register():
         carrier.Carrier,
         sale.Sale,
         module='carrier_api_ups', type_='model')
-    Pool.register(
-        checkout.Checkout,
-        depends=['nereid_webshop', 'nereid_checkout'],
-        module='carrier_api_ups', type_='model')
     Pool.register(
         sale.ApplyShipping,
         sale.GetRateUPS,
