@@ -73,34 +73,39 @@ class Sale(metaclass=PoolMeta):
             if not transaction.context.get('ups_rate_api_mode'):
                 context['ups_rate_api_mode'] = 'Shop'
             with transaction.set_context(**context):
-                rate_wizard = GetRateWizard(GetRateWizard.create()[0])
                 currency = Company(transaction.context['company']).currency
-                try:
-                    result = rate_wizard.transition_start()
-                except UPSError as e:
-                    return {
-                        'display_name': carrier.party.name,
-                        'cost': Decimal('0.0'),
-                        'cost_currency': currency,
-                        'carrier': carrier,
-                        'errors': str(e),
-                        }
-                service_types = dict(Carrier.ups_service_type.selection)
-                if not isinstance(result, list):
-                    # In case only one package is requested UPS returns
-                    # a dictionary instead of a list of one package
-                    result = [result]
-                for item in result:
-                    description = service_types[item['Service']['Code']]
-                    service = '%s - %s' % (carrier.party.name, description)
-                    rate = {
-                        'display_name': service,
-                        'cost': Decimal(item['TotalCharges']['MonetaryValue']),
-                        'cost_currency': currency,
-                        'carrier': carrier,
-                        'code': item['Service']['Code'],
-                        }
-                    rates.append(rate)
+                # As the wizard requires write access we start a new writable
+                # transaction, because this function can be called from
+                # readonly transactions like on_change* or web requests
+                with transaction.new_transaction():
+                    rate_wizard = GetRateWizard(GetRateWizard.create()[0])
+                    try:
+                        result = rate_wizard.transition_start()
+                    except UPSError as e:
+                        return {
+                            'display_name': carrier.party.name,
+                            'cost': Decimal('0.0'),
+                            'cost_currency': currency,
+                            'carrier': carrier,
+                            'errors': str(e),
+                            }
+                    service_types = dict(Carrier.ups_service_type.selection)
+                    if not isinstance(result, list):
+                        # In case only one package is requested UPS returns
+                        # a dictionary instead of a list of one package
+                        result = [result]
+                    for item in result:
+                        description = service_types[item['Service']['Code']]
+                        service = '%s - %s' % (carrier.party.name, description)
+                        rate = {
+                            'display_name': service,
+                            'cost': Decimal(
+                                item['TotalCharges']['MonetaryValue']),
+                            'cost_currency': currency,
+                            'carrier': carrier,
+                            'code': item['Service']['Code'],
+                            }
+                        rates.append(rate)
             return rates
 
         return super().get_shipping_rate(carrier)
